@@ -22,12 +22,14 @@
 #include "vtkSmartPointer.h"
 
 #include <array>
+#include <cstdint>
 #include <memory>
 #include <string>
 #include <vector>
 
 VTK_ABI_NAMESPACE_BEGIN
 class vtkOpenGLRenderWindow;
+class vtkOpenXRRenderWindow;
 
 class VTKRENDERINGOPENXR_EXPORT vtkOpenXRManager
 {
@@ -223,10 +225,10 @@ public:
   /**
    * Prepare the rendering resources for the specified eye and store in \p colorTextureId and
    * in \p depthTextureId (if the depth extension is supported) the texture in which we need
-   * to draw pixels.
+   * to draw pixels. \p win is used to get the right clipping planes for the depth extension.
    * Return true if no error occurred.
    */
-  bool PrepareRendering(uint32_t eye, void* colorTextureId, void* depthTextureId);
+  bool PrepareRendering(vtkOpenXRRenderWindow* win, void* colorTextureId, void* depthTextureId);
   ///@}
 
   ///@{
@@ -380,6 +382,26 @@ public:
   vtkOpenXRManagerConnection* GetConnectionStrategy() { return this->ConnectionStrategy; }
   ///@}
 
+  ///@{
+  /**
+   * Enable or disable XR_KHR_composition_layer_depth extension even when it is available.
+   *
+   * This must be set before vtkOpenXRManager initialization.
+   * Do nothing if XR_KHR_composition_layer_depth isn't available.
+   *
+   * Depth information is useful for augmented reality devices such as the Hololens 2.
+   * When enabled and XR_KHR_composition_layer_depth is available,
+   * the depth texture of the render window is submitted to the runtime.
+   * The runtime will use this information increase hologram stability of the Hololens 2.
+   *
+   * Note: enabling this option without providing the depth information could reduce stability.
+   *
+   * Default value: `false`
+   */
+  void SetUseDepthExtension(bool value) { this->UseDepthExtension = value; }
+  bool GetUseDepthExtension() const { return this->UseDepthExtension; }
+  ///@}
+
 protected:
   vtkOpenXRManager();
   ~vtkOpenXRManager() = default;
@@ -593,9 +615,13 @@ protected:
    */
   XrTime PredictedDisplayTime;
 
+  bool UseDepthExtension = false;
   bool SessionRunning = false;
-  // After each WaitAndBeginFrame, the OpenXR runtime may inform us that
-  // the current frame should not be rendered. Store it to avoid a render
+  // Following each WaitAndBeginFrame operation, the OpenXR runtime may indicate
+  // whether the current frame should be rendered using the `XrFrameState.shouldRender`
+  // property. We store this information to optimize rendering and prevent unnecessary
+  // render calls. For further details, refer to:
+  // https://registry.khronos.org/OpenXR/specs/1.0/man/html/XrFrameState.html
   bool ShouldRenderCurrentFrame = false;
   // If true, the function UpdateActionData will store
   // pose velocities for pose actions
